package com.work.provide_tool.util

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.graphics.Rect
import android.util.DisplayMetrics
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import android.widget.ImageView
import androidx.core.content.ContextCompat
import androidx.core.content.getSystemService
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.work.provide_tool.R
import com.work.provide_tool.app.IApp
import com.work.provide_tool.app.IApp.Companion.app
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.launch


//import kotlinx.coroutines.withContext


fun getAppColor(id:Int): Int {
    return ContextCompat.getColor(app,id)
}

fun getDrawIdentifier(idStr:String):Int{
    return app.resources.getIdentifier(idStr,"mipmap", app.packageName)
}

fun addListener(l:(View)->Unit,vararg view:View){
    view.forEach { view->
        view.setOnClickListener(l)
    }
}
var SHOW = 1
var HIDE = 2
fun showLayoutType(v:View,showTag:Int){
    if(showTag == SHOW){
        if(v.visibility != View.VISIBLE){
            v.visibility = View.VISIBLE
        }
    }else{
        if(v.visibility == View.VISIBLE){
            v.visibility = View.GONE
        }
    }
}

//keep 数据在刷ui的
fun keepDataUpdateWithUi(lfo: LifecycleOwner,otherThread:suspend ()->Unit,uiOfThread:()->Unit = {}){
    lfo.lifecycleScope.launch {
         lfo.lifecycleScope.async (Dispatchers.IO) {
            otherThread()
        }.await().apply {
            uiOfThread()
        }
    }
}


//-----------------------------------------Glide----------------------------------------------
var TPYER_CROP = "crop"
var TPYE_FIT = "fit"

//长方形
fun glideLoadRectangle(url: Any?,iv:ImageView?,tag: String = TPYER_CROP){
    glideGeneralOption(url,iv, getOptionGlide(tag, R.mipmap.icon_default_rectangle,R.mipmap.icon_default_rectangle))
}

//正方形
fun glideLoad(url: Any?,iv:ImageView?,tag: String = TPYER_CROP){
    glideGeneralOption(url,iv, getOptionGlide(tag, R.mipmap.icon_default,R.mipmap.icon_default))
}

//圆
fun glideLoadRound(url: Any?,iv:ImageView?,tag: String = TPYER_CROP){
    glideGeneralOption(url,iv, getOptionGlide(tag, R.mipmap.icon_default_round,R.mipmap.icon_default_round))
}

@SuppressLint("CheckResult")
private fun getOptionGlide(tag: String, placeholder:Int = 0, unload:Int = 0,doWork:RequestOptions.()->RequestOptions = {this}):RequestOptions{
    return RequestOptions().apply {
        when(tag){
            TPYER_CROP -> centerCrop()
            TPYE_FIT -> fitCenter()
            else -> centerCrop()
        }

        if(placeholder != 0){
            placeholder(placeholder)
        }

        if(unload != 0){
            error(unload)
        }
    }.doWork()
}

private fun glideGeneralOption(url:Any?,iv:ImageView?,requestOptions:RequestOptions){
    if(url == null){
        return
    }

    if(iv == null){
        return
    }

    when (url) {
        is String -> {
            url.run {
                if(isEmpty()){
                    return
                }

                if(startsWith("https")){
                    replace("https","http")
                }
            }
        }

        is Int -> {
            url.run {
                if(this == 0){
                    return
                }
            }
        }

        else -> {
            ToastUtil.show("glide load $url ??")
        }
    }


    Glide.with(app).load(url).apply(requestOptions).into(iv)
}
//-----------------------------------------/Glide----------------------------------------------

//-----------------------------------------屏宽高-----------------------------------------
/**
 *
 * @param tag 1:宽     2:高
 */
fun getTotalPhoneWithHW(tag: Int): Int {
    val displayMetrics: DisplayMetrics =  app.resources.displayMetrics
    val widthPixels = displayMetrics.widthPixels //当前屏幕宽
    val heightPixels = displayMetrics.heightPixels //当前屏幕高
    return if (tag == 1) widthPixels else heightPixels
}

fun dp2Px(context: Context, dp: Float): Int {
    val scale = context.resources.displayMetrics.density
    return (dp * scale + 0.5f).toInt()
}

fun px2Dp(context: Context, px: Float): Int {
    val scale = context.resources.displayMetrics.density
    return (px / scale + 0.5f).toInt()
}

fun px2sp(context: Context, pxValue: Float): Int {
    val fontScale = context.resources.displayMetrics.scaledDensity
    return (pxValue / fontScale + 0.5f).toInt()
}

fun sp2Px(context: Context, spValue: Float): Int {
    val fontScale = context.resources.displayMetrics.scaledDensity
    return (spValue * fontScale + 0.5f).toInt()
}
//-----------------------------------------/屏宽高-----------------------------------------

//-----------------------------------------打开软键盘-----------------------------------------
fun showInputKey(editText: EditText){
    editText.requestFocus()
    var i:InputMethodManager = app.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
    i.showSoftInput(editText, 0)
}

fun hideInputKey(editText:EditText){
    var i:InputMethodManager = app.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
    i.hideSoftInputFromWindow(editText.windowToken,0)
}

fun getInputKey(a:Activity): Boolean {
    var r = Rect()
    a.window.decorView.getWindowVisibleDisplayFrame(r)
    var v = r.bottom
    return getTotalPhoneWithHW(2)- v > 0
}
//-----------------------------------------/打开软键盘-----------------------------------------